/*
** Module : StreamBuilder.java
** Public : interface for remote construction of streams
**
** Copyright (c) 2006-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 GES 20060211  @24552 Created initial version providing remote construction of streams.
** 002 CS  20130103         Implement OS-DIR support as an INPUT stream.
** 003 VIG 20130918         Added printer stream support (OUTPUT TO PRINTER).
** 004 GES 20140612         Added explicit terminal stream support.
*/ 

package com.goldencode.p2j.util;

import java.io.*;

/**
 * An interface providing remote construction of file and process streams.
 */
public interface StreamBuilder
{
   /**
    * Constructs a remote stream instance representing a file using a filename, this file or
    * device will be opened for reading or writing based on the given <code>write</code> flag.
    * The current read/write position will be at offset 0 unless in <code>write</code> mode and 
    * the <code>append</code> flag is on (in which case the write position will be at the end of
    * the file).
    * <p>
    * This stream will NOT have paging active.
    *
    * @param    filename
    *           The name of the file or device to open.  Must not be <code>null</code>.
    * @param    write
    *           If <code>true</code>, open in write mode, otherwise open the file for reading.
    * @param    append
    *           In <code>write</code> mode, if <code>true</code>, the current file pointer is
    *           set to the end of the file. Otherwise if in <code>write</code> mode, the file
    *           is truncated to 0 length. This value is ignored in <code>read</code> mode.
    *           
    * @return   The stream id which is used to route operations to the newly created instance.
    *
    * @throws   ErrorConditionException
    *           If the file does not exist OR if the current user does not have the proper rights
    *           to open the file or device.
    */
   public int openFileStream(String filename, boolean write, boolean append)
   throws ErrorConditionException;
      
   /**
    * Constructs a remote stream instance representing a process' standard I/O.
    *
    * @return   The stream id which is used to route operations to the newly created instance.
    */
   public int openProcessStream();

   /**
    * Constructs a remote stream instance representing the OsDirStream result of the INPUT 
    * THROUGH OS-DIR (dir-name) statement.
    * <p>
    * This will take the directory and export the list of files, directory and device names
    * and properties.
    * 
    * @param    dir
    *           The <code>String</code> path to the directory on which OS-DIR will be applied.
    * @param    noAttrList
    *           <code>true</code> if property indexes will be added to the results.
    *        
    * @return   The stream id which is used to route operations to the newly created instance.
    */
   public int openDirStream(String dir, boolean noAttrList);

   /**
    * Constructs a remote stream instance representing the terminal.
    *
    * @return   The stream id which is used to route operations to the newly created instance.
    */
   public int openTerminalStream();
   
   /**
    * Constructs a remote printer stream instance as the unnamed output (OUTPUT TO PRINTER
    * statement).
    * 
    * @return   The stream id which is used to route operations to the newly created instance.
    */
   public int openPrinterStream();

   /**
    * Constructs a remote printer stream instance.  This can be assigned as unnamed output
    * (OUTPUT TO PRINTER statement) or as a named stream (OUTPUT STREAM someStream TO PRINTER).
    * 
    * @param    name
    *           The name of the target printer. 
    * 
    * @return   The stream id which is used to route operations to the newly created instance.
    */
   public int openPrinterStream(String name);
}
